@@ -73,23 +73,19 @@
 #if CONFIG_XFREE86_VERSION < XFREE86_VERSION(4,1,0,0)
 #define DRM_PROC_DEVICES "/proc/devices"
 #define DRM_PROC_MISC	 "/proc/misc"
-#define DRM_PROC_DRM	 "/proc/drm"
-#define DRM_DEV_DRM	 "/dev/drm"
+#define DRM_PROC_DRM	 "/proc/ati"
+#define DRM_DEV_DRM	 "/dev/ati"
 #define DRM_DEV_MODE	 (S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP)
 #define DRM_DEV_UID	 0
 #define DRM_DEV_GID	 0
 #endif
 
 #if CONFIG_XFREE86_VERSION >= XFREE86_VERSION(4,1,0,0)
-#ifdef __OpenBSD__
-#define DRM_MAJOR       81
-#endif
-#if defined(__linux__) || defined(__NetBSD__)
-#define DRM_MAJOR       226
-#endif
+/* FGLRX uses a dynamically allocated major device number. Therefore
+ * DRM_MAJOR is not defined. */
 #define DRM_MAX_MINOR   15
 #endif
-#define DRM_NAME	"drm"	  /**< Name in kernel, /dev, and /proc */
+#define DRM_NAME	"ati"	  /**< Name in kernel, /dev, and /proc */
 #define DRM_MIN_ORDER	5	  /**< At least 2^5 bytes = 32 bytes */
 #define DRM_MAX_ORDER	22	  /**< Up to 2^22 bytes = 4MB */
 #define DRM_RAM_PERCENT 10	  /**< How much system ram can we lock? */
@@ -270,6 +266,26 @@ typedef struct drm_map {
 				 /*   Private data */
 } drm_map_t;
 
+/**
+ * drm_map_extended_t is used to support direct fb mapping over 4GB on 32bit OS.
+ */
+typedef struct drm_map_extended {
+        unsigned long long offset;
+        unsigned long long size;
+        drm_map_type_t     type;
+        drm_map_flags_t    flags;
+        void               *handle;
+        int                mtrr;
+} drm_map_extended_t;
+
+#define COPY_DRMMAP_FIELDS(_to, _from) \
+    _to.offset = _from.offset; \
+    _to.size = _from.size; \
+    _to.type = _from.type; \
+    _to.flags = _from.flags; \
+    _to.handle = _from.handle; \
+    _to.mtrr = _from.mtrr;
+
 
 /**
  * DRM_IOCTL_GET_CLIENT ioctl argument type.
