@@ -20,6 +20,7 @@
 #include <stdarg.h>
 #include "kcl_pci.h"
 #include "kcl_io.h"
+#include "kcl_iommu.h"
 
 #define FIREGL_USWC_SUPPORT     1
 
@@ -178,51 +179,34 @@ struct semaphore;
 struct drm_device;
 struct firegl_pcie_mem;
 
-/*****************************************************************************/
-
-typedef unsigned long __ke_dev_t;
-typedef unsigned long __ke_off_t;
-#ifdef __x86_64__
-typedef long __ke_ssize_t;
-typedef unsigned long __ke_size_t;
-#else
-typedef int __ke_ssize_t;
-typedef unsigned int __ke_size_t;
-#endif
-typedef unsigned char __ke_u8;
-typedef unsigned short __ke_u16;
-typedef unsigned int __ke_u32;
-typedef unsigned long long __ke_u64;
-typedef unsigned long long __ke_dma_addr_t;
-typedef long long __ke_loff_t;
 
 // note: assigning uniqe types to originally non interchangeable types
-typedef struct { int uniqe3; } __ke_sigset_t;
-typedef struct { int uniqe7; } __ke_file_operations_t;
+typedef struct { int uniqe3; } kcl_sigset_t;
+typedef struct { int uniqe7; } kcl_file_operations_t;
 
-typedef	int (*__ke_read_proc_t)(
-    char* page, char** start, __ke_off_t off, int count, int* eof, void* data);
+typedef	int (*kcl_read_proc_t)(
+    char* page, char** start, kcl_off_t off, int count, int* eof, void* data);
 
 typedef struct {
     const char*             name;
-    __ke_read_proc_t        f;
-    __ke_file_operations_t* fops;
-} __ke_proc_list_t;
+    kcl_read_proc_t        f;
+    kcl_file_operations_t* fops;
+} kcl_proc_list_t;
 
-extern __ke_proc_list_t KCL_PROC_FileList[];
+extern kcl_proc_list_t KCL_PROC_FileList[];
 
 typedef struct {
     unsigned long           signature;
     int                     privdevcount; // number of privdev structs
-    __ke_proc_list_t *      proclist;
+    kcl_proc_list_t *      proclist;
     const char *            name;
 	unsigned int	        major_version;
 	unsigned int	        minor_version;
 	unsigned int            patchlevel;
 	const char *	        date;
-    __ke_sigset_t *         psigmask;
+    kcl_sigset_t *         psigmask;
     void *                  privglobal; // private global context
-} __ke_device_t;
+} kcl_device_t;
 
 /*****************************************************************************/
 
@@ -361,41 +345,41 @@ extern int           ATI_API_CALL KCL_PosixSecurityCapCheck(KCL_ENUM_PosixSecuri
 
 /*****************************************************************************/
 
-extern int ATI_API_CALL drm_name_info(char* buf, char** start, __ke_off_t offset, int len, int* eof, void* data);
-extern int ATI_API_CALL firegl_bios_version(char* buf, char** start, __ke_off_t offset, int len, int* eof, void* data);
-extern int ATI_API_CALL firegl_interrupt_info(char* buf, char** start, __ke_off_t offset, int len, int* eof, void* data);
-extern int ATI_API_CALL drm_mem_info(char* buf, char** start, __ke_off_t offset, int len, int *eof, void *data);
-extern int ATI_API_CALL drm_mem_info1(char* buf, char** start, __ke_off_t offset, int len, int *eof, void *data);
-extern int ATI_API_CALL drm_vm_info(char* buf, char** start, __ke_off_t offset, int len, int* eof, void* data);
-extern int ATI_API_CALL drm_clients_info(char* buf, char** start, __ke_off_t offset, int len, int* eof, void* data);
-extern int ATI_API_CALL firegl_lock_info(char* buf, char** start, __ke_off_t offset, int len, int* eof, void* data);
-extern int ATI_API_CALL firegl_ptm_info(char* buf, char** start, __ke_off_t offset, int len, int *eof, void *data);
+extern int ATI_API_CALL drm_name_info(char* buf, char** start, kcl_off_t offset, int len, int* eof, void* data);
+extern int ATI_API_CALL firegl_bios_version(char* buf, char** start, kcl_off_t offset, int len, int* eof, void* data);
+extern int ATI_API_CALL firegl_interrupt_info(char* buf, char** start, kcl_off_t offset, int len, int* eof, void* data);
+extern int ATI_API_CALL drm_mem_info(char* buf, char** start, kcl_off_t offset, int len, int *eof, void *data);
+extern int ATI_API_CALL drm_mem_info1(char* buf, char** start, kcl_off_t offset, int len, int *eof, void *data);
+extern int ATI_API_CALL drm_vm_info(char* buf, char** start, kcl_off_t offset, int len, int* eof, void* data);
+extern int ATI_API_CALL drm_clients_info(char* buf, char** start, kcl_off_t offset, int len, int* eof, void* data);
+extern int ATI_API_CALL firegl_lock_info(char* buf, char** start, kcl_off_t offset, int len, int* eof, void* data);
+extern int ATI_API_CALL firegl_ptm_info(char* buf, char** start, kcl_off_t offset, int len, int *eof, void *data);
 #ifdef DEBUG
-extern int ATI_API_CALL drm_bq_info(char* buf, char** start, __ke_off_t offset, int len, int* eof, void* data);
+extern int ATI_API_CALL drm_bq_info(char* buf, char** start, kcl_off_t offset, int len, int* eof, void* data);
 #endif
 extern int ATI_API_CALL firegl_interrupt_open(void* data, KCL_IO_FILE_Handle file);
 extern int ATI_API_CALL firegl_interrupt_release(KCL_IO_FILE_Handle file);
 extern unsigned int ATI_API_CALL firegl_interrupt_read(
                                     KCL_IO_FILE_Handle user_file, 
                                     char *user_buf, 
-                                    __ke_size_t user_buf_size, 
-                                    __ke_loff_t *user_file_pos);
+                                    kcl_size_t user_buf_size, 
+                                    kcl_loff_t *user_file_pos);
 extern unsigned int ATI_API_CALL firegl_interrupt_poll(
         KCL_IO_FILE_Handle user_file, KCL_IO_FILE_PollTableHandle pt);
 extern int ATI_API_CALL firegl_interrupt_write(
                                     KCL_IO_FILE_Handle user_file,
                                     const char *user_buf, 
-                                    __ke_size_t user_buf_size, 
-                                    __ke_loff_t *user_file_pos);
+                                    kcl_size_t user_buf_size, 
+                                    kcl_loff_t *user_file_pos);
 
-extern int ATI_API_CALL firegl_debug_proc_read(char* buf, char** start, __ke_off_t offset, int len, int* eof, void* data);
+extern int ATI_API_CALL firegl_debug_proc_read(char* buf, char** start, kcl_off_t offset, int len, int* eof, void* data);
 extern int ATI_API_CALL firegl_debug_proc_write(void* file, const char *buffer, unsigned long count, void *data); 
 
 /*****************************************************************************/
 
-extern int ATI_API_CALL firegl_private_init (__ke_device_t *);
-extern void ATI_API_CALL firegl_private_cleanup (__ke_device_t *);
-extern int ATI_API_CALL firegl_init(__ke_device_t*);
+extern int ATI_API_CALL firegl_private_init (kcl_device_t *);
+extern void ATI_API_CALL firegl_private_cleanup (kcl_device_t *);
+extern int ATI_API_CALL firegl_init(kcl_device_t*);
 extern int ATI_API_CALL firegl_open(int minor, KCL_IO_FILE_Handle filp);
 extern int ATI_API_CALL firegl_release(KCL_IO_FILE_Handle filp);
 extern int ATI_API_CALL firegl_ioctl(
@@ -427,30 +411,30 @@ extern const unsigned int* ATI_API_CALL mc_heap_get_page_idx_list(void *pointer_
 
 /*****************************************************************************/
 
-extern void ATI_API_CALL KCL_SIGNAL_ConfigureMask(__ke_sigset_t *pSigMask);
+extern void ATI_API_CALL KCL_SIGNAL_ConfigureMask(kcl_sigset_t *pSigMask);
 extern int ATI_API_CALL firegl_sig_notifier(void *priv);
 extern int              KCL_SIGNAL_Notifier(void *priv);
-extern void ATI_API_CALL KCL_SIGNAL_BlockAll(int (*notifier)(void *priv), void *pPriv, __ke_sigset_t *pSigMask);
+extern void ATI_API_CALL KCL_SIGNAL_BlockAll(int (*notifier)(void *priv), void *pPriv, kcl_sigset_t *pSigMask);
 extern void ATI_API_CALL KCL_SIGNAL_UnblockAll(void);
 
-extern unsigned long ATI_API_CALL __ke__cmpxchg(volatile void *ptr, unsigned long old,                      
+extern unsigned long ATI_API_CALL kcl__cmpxchg(volatile void *ptr, unsigned long old,                      
                       unsigned long new, int size);
 
-#define __ke_cmpxchg(ptr,o,n)                        \
-  ((__typeof__(*(ptr)))__ke__cmpxchg((ptr),(unsigned long)(o),      \
+#define kcl_cmpxchg(ptr,o,n)                        \
+  ((__typeof__(*(ptr)))kcl__cmpxchg((ptr),(unsigned long)(o),      \
                  (unsigned long)(n),sizeof(*(ptr))))
 /*****************************************************************************/
 
-extern __ke_dev_t ATI_API_CALL KCL_DEVICE_GetNumber(__ke_device_t *dev);
+extern unsigned int ATI_API_CALL KCL_DEVICE_GetNumber(kcl_device_t *dev);
 
 extern void ATI_API_CALL KCL_MODULE_IncUseCount(void);
 extern void ATI_API_CALL KCL_MODULE_DecUseCount(void);
 
-extern void ATI_API_CALL KCL_SEMAPHORE_STATIC_Down(__ke_device_t *dev, int idx);
-extern void ATI_API_CALL KCL_SEMAPHORE_STATIC_Up(__ke_device_t *dev, int idx);
-#define __KE_MAX_SEMAPHORES 4
+extern void ATI_API_CALL KCL_SEMAPHORE_STATIC_Down(kcl_device_t *dev, int idx);
+extern void ATI_API_CALL KCL_SEMAPHORE_STATIC_Up(kcl_device_t *dev, int idx);
+#define __KE_MAX_SEMAPHORES 3
 extern void ATI_API_CALL KCL_SEMAPHORE_Init(struct semaphore* sem, int value);
-extern __ke_size_t ATI_API_CALL KCL_SEMAPHORE_GetObjSize(void);
+extern kcl_size_t ATI_API_CALL KCL_SEMAPHORE_GetObjSize(void);
 extern void ATI_API_CALL KCL_SEMAPHORE_DownUninterruptible(struct semaphore* sem);
 extern void ATI_API_CALL KCL_SEMAPHORE_Up(struct semaphore* sem);
 
@@ -458,22 +442,26 @@ extern void ATI_API_CALL KCL_SEMAPHORE_Up(struct semaphore* sem);
 extern int ATI_API_CALL KCL_SEMAPHORE_DownInterruptible(struct semaphore* sem);
 //PPLIB end
 
-extern void ATI_API_CALL KCL_SPINLOCK_STATIC_Grab(__ke_device_t *dev, int ndx);
-extern void ATI_API_CALL KCL_SPINLOCK_STATIC_Release(__ke_device_t *dev, int ndx);
-	#define __KE_MAX_SPINLOCKS 8
+extern void ATI_API_CALL KCL_SPINLOCK_STATIC_Grab(kcl_device_t *dev, int ndx);
+extern void ATI_API_CALL KCL_SPINLOCK_STATIC_Release(kcl_device_t *dev, int ndx);
+#ifdef VCE_SUPPORT
+#define __KE_MAX_SPINLOCKS 9
+#else
+#define __KE_MAX_SPINLOCKS 8 
+#endif
 
-int ATI_API_CALL __ke_vsprintf(char *buf, const char *fmt, va_list ap);
-int ATI_API_CALL __ke_vsnprintf(char *buf, size_t size, const char *fmt, va_list ap);
+int ATI_API_CALL kcl_vsprintf(char *buf, const char *fmt, va_list ap);
+int ATI_API_CALL kcl_vsnprintf(char *buf, size_t size, const char *fmt, va_list ap);
 
-extern int ATI_API_CALL KCL_CopyFromUserSpace(void* to, const void* from, __ke_size_t size);
-extern int ATI_API_CALL KCL_CopyToUserSpace(void* to, const void* from, __ke_size_t size);
+extern int ATI_API_CALL KCL_CopyFromUserSpace(void* to, const void* from, kcl_size_t size);
+extern int ATI_API_CALL KCL_CopyToUserSpace(void* to, const void* from, kcl_size_t size);
 
-extern void* ATI_API_CALL KCL_MEM_SmallBufferAlloc(__ke_size_t size);
-extern void* ATI_API_CALL KCL_MEM_SmallBufferAllocAtomic(__ke_size_t size);
+extern void* ATI_API_CALL KCL_MEM_SmallBufferAlloc(kcl_size_t size);
+extern void* ATI_API_CALL KCL_MEM_SmallBufferAllocAtomic(kcl_size_t size);
 extern void ATI_API_CALL KCL_MEM_SmallBufferFree(void* p);
-extern void* ATI_API_CALL __ke_vmalloc(__ke_size_t size);
-extern void* ATI_API_CALL KCL_MEM_Alloc(__ke_size_t size);
-extern void* ATI_API_CALL KCL_MEM_AllocAtomic(__ke_size_t size);
+extern void* ATI_API_CALL kcl_vmalloc(kcl_size_t size);
+extern void* ATI_API_CALL KCL_MEM_Alloc(kcl_size_t size);
+extern void* ATI_API_CALL KCL_MEM_AllocAtomic(kcl_size_t size);
 extern void ATI_API_CALL KCL_MEM_Free(void* p);
 extern void* ATI_API_CALL KCL_MEM_AllocPageFrame(void);
 extern void* ATI_API_CALL KCL_MEM_AllocContiguousPageFrames(int order);
@@ -486,8 +474,8 @@ extern void ATI_API_CALL KCL_MEM_FreePageForGart(void* pt);
 extern void ATI_API_CALL KCL_MEM_IncPageUseCount(void* pt);
 extern void ATI_API_CALL KCL_MEM_DecPageUseCount(void* pt);
 extern void ATI_API_CALL KCL_MEM_IncPageCount_Mapping(void* page);
-extern int ATI_API_CALL KCL_MEM_VerifyReadAccess(void* addr, __ke_size_t size);
-extern int ATI_API_CALL KCL_MEM_VerifyWriteAccess(void* addr, __ke_size_t size);
+extern int ATI_API_CALL KCL_MEM_VerifyReadAccess(void* addr, kcl_size_t size);
+extern int ATI_API_CALL KCL_MEM_VerifyWriteAccess(void* addr, kcl_size_t size);
 extern unsigned long ATI_API_CALL KCL_GetPageTableByVirtAddr(unsigned long virtual_addr, unsigned long* page_addr);
 extern int ATI_API_CALL KCL_LockUserPages(unsigned long vaddr, unsigned long* page_list, unsigned int page_cnt);
 extern void ATI_API_CALL KCL_UnlockUserPages(unsigned long* page_list, unsigned int page_cnt);
@@ -530,7 +518,7 @@ extern void* ATI_API_CALL KCL_MEM_VM_GetRegionPrivateData(struct vm_area_struct*
 extern unsigned long ATI_API_CALL KCL_MEM_VM_GetRegionStart(struct vm_area_struct* vma);
 extern unsigned long ATI_API_CALL KCL_MEM_VM_GetRegionEnd(struct vm_area_struct* vma);
 extern unsigned long ATI_API_CALL KCL_MEM_VM_GetRegionMapOffset(struct vm_area_struct* vma);
-enum __ke_vm_maptype
+enum kcl_vm_maptype
 {
     __KE_ADPT,
     __KE_SHM,
@@ -549,11 +537,11 @@ extern char* ATI_API_CALL KCL_MEM_VM_GetRegionProtFlagsStr(struct vm_area_struct
 extern char* ATI_API_CALL KCL_MEM_VM_GetRegionPhysAddrStr(struct vm_area_struct* vma,
                                    char* buf, 
                                    unsigned long linear_address, 
-                                   __ke_dma_addr_t* phys_address);
+                                   kcl_dma_addr_t* phys_address);
 extern int ATI_API_CALL KCL_MEM_VM_MapRegion(KCL_IO_FILE_Handle filp,
                                     struct vm_area_struct* vma,
                                     unsigned long long offset,
-                                    enum __ke_vm_maptype type,
+                                    enum kcl_vm_maptype type,
                                     int readonly,
                                     void *private_data);
 
@@ -561,7 +549,7 @@ extern int ATI_API_CALL KCL_MEM_VM_MapRegion(KCL_IO_FILE_Handle filp,
 
 extern int ATI_API_CALL firegl_pci_save_state(KCL_PCI_DevHandle pdev, struct drm_device* dev);
 extern int ATI_API_CALL firegl_pci_restore_state(KCL_PCI_DevHandle pdev, struct drm_device* dev);
-extern int ATI_API_CALL firegl_cail_powerdown(struct drm_device* dev, __ke_u32 state);
+extern int ATI_API_CALL firegl_cail_powerdown(struct drm_device* dev, int state);
 extern int ATI_API_CALL firegl_cail_powerup(struct drm_device* dev);
 extern void ATI_API_CALL firegl_pm_save_framebuffer(struct drm_device* dev);
 extern void ATI_API_CALL firegl_pm_restore_framebuffer(struct drm_device* dev);
@@ -574,7 +562,7 @@ extern void ATI_API_CALL firegl_pm_lock_highmem_gart(struct drm_device* dev, int
 
 /*****************************************************************************/
 
-extern int ATI_API_CALL KCL_PM_Is_SuspendToRam(unsigned int state);
+extern int ATI_API_CALL KCL_PM_Is_SuspendToRam(int state);
 
 /*****************************************************************************/
 
@@ -790,19 +778,19 @@ typedef enum {
     __KE_SIGIO,
     __KE_ESPIPE,
     __KE_EINTR
-} __ke_asynio_contant_t;
+} kcl_asynio_contant_t;
 
 extern int ATI_API_CALL KCL_SYSINFO_MapConstant(int contant);
 
-extern __ke_ssize_t ATI_API_CALL firegl_asyncio_read( KCL_IO_FILE_Handle filp,
+extern kcl_ssize_t ATI_API_CALL firegl_asyncio_read( KCL_IO_FILE_Handle filp,
                                                       char *buf, 
-                                                      __ke_size_t size,
-                                                      __ke_loff_t *off_ptr);
+                                                      kcl_size_t size,
+                                                      kcl_loff_t *off_ptr);
 
-extern __ke_ssize_t ATI_API_CALL firegl_asyncio_write( KCL_IO_FILE_Handle filp,
+extern kcl_ssize_t ATI_API_CALL firegl_asyncio_write( KCL_IO_FILE_Handle filp,
                                                        const char *buf, 
-                                                       __ke_size_t size,
-                                                       __ke_loff_t *off_ptr);
+                                                       kcl_size_t size,
+                                                       kcl_loff_t *off_ptr);
 
 extern unsigned int ATI_API_CALL firegl_asyncio_poll(
         KCL_IO_FILE_Handle filp, KCL_IO_FILE_PollTableHandle table);
@@ -837,4 +825,13 @@ extern int ATI_API_CALL firegl_init_32compat_ioctls(void);
 extern void ATI_API_CALL firegl_kill_32compat_ioctls(void);
 extern int ATI_API_CALL firegl_uswc_user_disabled(void);
 
+//export function prototype from libip
+extern int libip_iommu_invalid_pri_request( KCL_PCI_DevHandle pdev,
+                                            int  pasid,
+                                            unsigned long fault_addr,
+                                            KCL_IOMMU_req_perm_t perm);
+
+extern int libip_iommu_vas_term( KCL_PCI_DevHandle pdev, int  pasid);
+
+
 #endif /* _FIREGL_PUBLIC_H_ */
