@@ -15,8 +15,17 @@
  ****************************************************************************/
 
 #include <linux/version.h>
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,33)
+#include <generated/autoconf.h>
+#else
 #include <linux/autoconf.h>
+#endif
 #include <linux/acpi.h>
+#include <linux/pci.h>
+
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,32)
+#include <acpi/button.h>
+#endif
 
 #include "kcl_config.h"
 #include "kcl_type.h"
@@ -39,8 +48,22 @@
 
 #endif
 
+/*
+ * NOTE: When adding new KCL ACPI functions below, remember to add an appropriate
+ *       stub function in the #else part of KCL_ACPI_CONFIGURED to handle those
+ *       cases where ACPI support is not built within the kernel.  If the function
+ *       should be implemented the same way regardless of whether ACPI support is
+ *       available, put it after the #endif KCL_ACPI_CONFIGURED section by the end
+ *       of the file.
+ */
+
 #if defined(KCL_ACPI_CONFIGURED)
 
+struct firegl_acpi_pci_data {
+    struct acpi_pci_id      id;
+    struct pci_bus          *bus;
+    struct pci_dev          *dev;
+};
 
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,29)
 
@@ -110,8 +133,23 @@ static struct notifier_block firegl_acpi_ac_notifier = {
         .notifier_call = firegl_acpi_ac_event,
 };
 
-#endif
 
+static int firegl_acpi_lid_event(struct notifier_block *nb, unsigned long val,
+                               void *data)
+{
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,32)
+    if (!acpi_lid_open()) 
+    {	
+        libip_lid_open_notify();
+    }
+#endif
+    return NOTIFY_OK;
+}
+
+static struct notifier_block firegl_acpi_lid_notifier = {
+        .notifier_call = firegl_acpi_lid_event,
+};
+#endif
 /** \brief search for acpi namespace node with specified name
  *  \param parent specify the search scope
  *  \param pathname pointer to a null terminated ascii string
@@ -125,17 +163,6 @@ unsigned int ATI_API_CALL KCL_ACPI_GetDevHandle(KCL_ACPI_DevHandle parent,
     return acpi_get_handle(parent, (acpi_string)pathname, ret_handle);
 }
 
-/** \brief Search for specified devices
- *  \param callback called when a matching device is found
- *  \param context passed to callback
- *  \return status
- */
-unsigned int ATI_API_CALL KCL_ACPI_GetDevices(KCL_ACPI_CallbackHandle callback,
-                                              KCL_ACPI_ContextHandle context)
-{
-    return acpi_get_devices(NULL, (acpi_walk_callback)callback, context, NULL);
-}
-
 /** \brief evaluate acpi namespace object, handle or pathname must be valid
  *  \param handle namespace object handle
  *  \param info input/output arguments for the control method
@@ -145,15 +172,14 @@ unsigned int ATI_API_CALL KCL_ACPI_EvalObject(KCL_ACPI_DevHandle handle,
                                               struct KCL_ACPI_MethodInputInfo *info)
 {
     struct acpi_object_list input;
-    struct acpi_buffer output;
+    struct acpi_buffer output = { ACPI_ALLOCATE_BUFFER, NULL };
     union acpi_object *params = NULL;
     union acpi_object *obj = NULL;
-    KCL_ACPI_DevHandle child_handle, next_child_handle;
     char name[5] = {'\0'};
     struct KCL_ACPI_MethodArgument *argument = NULL;
     unsigned int i, count;
-    unsigned int type = ACPI_TYPE_ANY;
     acpi_status status;
+    unsigned int funcNo = 0xFFFFFFFF;
 
     if (handle == NULL)
     {
@@ -161,7 +187,6 @@ unsigned int ATI_API_CALL KCL_ACPI_EvalObject(KCL_ACPI_DevHandle handle,
     }
 
     memset(&input, 0, sizeof(struct acpi_object_list));
-    memset(&output, 0, sizeof(struct acpi_buffer));
 
     /* validate input info */
     if (info->size != sizeof(struct KCL_ACPI_MethodInputInfo))
@@ -177,6 +202,7 @@ unsigned int ATI_API_CALL KCL_ACPI_EvalObject(KCL_ACPI_DevHandle handle,
             return KCL_ACPI_ERROR;
         }
         argument = info->pInputArgument;
+        funcNo = argument->value;
         for (i = 0; i < info->inputCount; i++)
         {
             if (((argument->type == ACPI_TYPE_STRING) || (argument->type == ACPI_TYPE_BUFFER))
@@ -253,84 +279,14 @@ unsigned int ATI_API_CALL KCL_ACPI_EvalObject(KCL_ACPI_DevHandle handle,
     count = info->outputCount;
     argument = info->pOutputArgument;
 
-    if (count > 1)
-    {
-        type = ACPI_TYPE_PACKAGE;
-    }
-    else if (count == 1)
-    {
-        type = argument->type;
-    }
-    else
-    {
-        type = ACPI_TYPE_INTEGER;
-    }
-
-    switch(type)
-    {
-        case ACPI_TYPE_INTEGER:
-            obj = (union acpi_object *)kmalloc(sizeof(union acpi_object), GFP_KERNEL);
-            if (obj == NULL)
-            {
-                status = KCL_ACPI_ERROR;
-                goto error;
-            }
-            memset(obj, 0, sizeof(union acpi_object));
-            output.length = sizeof(union acpi_object);
-            output.pointer = obj;
-            break;
-        case ACPI_TYPE_STRING:
-        case ACPI_TYPE_BUFFER:
-        case ACPI_TYPE_PACKAGE:
-            output.length = ACPI_ALLOCATE_BUFFER;
-            output.pointer = NULL;
-            break;
-        default:
-            output.length = 0;
-            output.pointer = NULL;
-            break;
-    }
-
     /* evaluate the acpi method */
     status = acpi_evaluate_object(handle, name, &input, &output);
+    KCL_DEBUG1(FN_FIREGL_ACPI, "Evaluated method: %s, handle: %p, function: %d, status: 0x%x  \n", name, handle, funcNo, (unsigned int)status);
+
     if (ACPI_FAILURE(status))
     {
-        /* if failed, try the child namespace node */
-        child_handle = NULL;
-        status = acpi_get_next_object (ACPI_TYPE_ANY, handle, child_handle, &next_child_handle);
-        while (ACPI_SUCCESS(status))
-        {
-            child_handle = next_child_handle;
-            switch(type)
-            {
-                case ACPI_TYPE_INTEGER:
-                    output.length = sizeof(union acpi_object);
-                    output.pointer = obj;
-                    break;
-                case ACPI_TYPE_STRING:
-                case ACPI_TYPE_BUFFER:
-                case ACPI_TYPE_PACKAGE:
-                    output.length = ACPI_ALLOCATE_BUFFER;
-                    output.pointer = NULL;
-                    break;
-                default:
-                    output.length = 0;
-                    output.pointer = NULL;
-                    break;
-            }
-
-            status = acpi_evaluate_object(child_handle, name, &input, &output);
-            if (ACPI_SUCCESS(status))
-            {
-                break;
-            }
-            status = acpi_get_next_object (ACPI_TYPE_ANY, handle, child_handle, &next_child_handle);
-        }
-        if (ACPI_FAILURE(status))
-        {
-            status = KCL_ACPI_ERROR;
-            goto error;
-        }
+        status = KCL_ACPI_ERROR;
+        goto error;
     }
 
     /* return the output info */
@@ -466,6 +422,28 @@ unsigned int ATI_API_CALL KCL_ACPI_RemoveHandler(KCL_ACPI_DevHandle device,
 #endif
 }
 
+unsigned int ATI_API_CALL KCL_ACPI_InstallLidHandler(void)
+{
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,32)
+    if (acpi_lid_notifier_register(&firegl_acpi_lid_notifier))
+    {
+        KCL_DEBUG_ERROR("lid notifier registration failed\n");
+        firegl_acpi_lid_notifier.notifier_call = NULL;
+    }
+#endif
+    return 0;
+}
+
+unsigned int ATI_API_CALL KCL_ACPI_RemoveLidHandler(void)
+{
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,32)
+    if (firegl_acpi_lid_notifier.notifier_call)
+    {
+        acpi_lid_notifier_unregister(&firegl_acpi_lid_notifier);
+    }
+#endif
+    return 0;
+}
 /** \brief get the global variable acpi_disabled in the kernel
  *  \return the value of acpi_disabled
  */
@@ -474,18 +452,6 @@ int ATI_API_CALL KCL_ACPI_Disabled(void)
     return acpi_disabled;
 }
 
-/** \brief user function passed to acpi_get_devices, and called when a device object found
- *  \param handle acpi handle
- *  \param level level of depth to search the namespace tree
- *  \param context context passed from acpi_get_devices
- *  \param retval where to put the return value
- *  \return status
- */
-unsigned int KCL_ACPI_GetAcpiHandles(KCL_ACPI_DevHandle handle, unsigned int level, KCL_ACPI_ContextHandle context, void **retval)
-{
-    return libip_get_acpi_handles(handle, level, context);
-}
-
 /** \brief a function of acpi notify handler type, called when state of ac adapter changed
  *  \param handle      acpi handle
  *  \param event       acpi event type
@@ -518,7 +484,7 @@ void ATI_API_CALL KCL_ACPI_ExecHandler(KCL_ACPI_CallbackHandle handler, KCL_ACPI
  */
 KCL_ACPI_CallbackHandle ATI_API_CALL KCL_ACPI_GetNotifyHandler(KCL_ACPI_DevHandle handle)
 {
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,29)
+#if LINUX_VERSION_CODE > KERNEL_VERSION(2,6,9) && LINUX_VERSION_CODE < KERNEL_VERSION(2,6,29)
     struct acpi_namespace_node *node = NULL;
 
     node = (struct acpi_namespace_node *)handle;
@@ -538,7 +504,9 @@ KCL_ACPI_CallbackHandle ATI_API_CALL KCL_ACPI_GetNotifyHandler(KCL_ACPI_DevHandl
 #endif
     /* No alternative on 2.6.29 and later for now. The internal ACPI data
      * structures are no longer included in the kernel headers. 
-     * For 2.6.29 and later, this is not used. Return null here.   */
+     * For 2.6.29 and later, this is not used. Return null here.
+     * And for 2.6.9, there is no ACPI video module, so also return null
+     */
     {
         return NULL;
     }
@@ -550,7 +518,7 @@ KCL_ACPI_CallbackHandle ATI_API_CALL KCL_ACPI_GetNotifyHandler(KCL_ACPI_DevHandl
  */
 KCL_ACPI_ContextHandle ATI_API_CALL KCL_ACPI_GetNotifyContext(KCL_ACPI_DevHandle handle)
 {
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,29)
+#if LINUX_VERSION_CODE > KERNEL_VERSION(2,6,9) && LINUX_VERSION_CODE < KERNEL_VERSION(2,6,29)
     struct acpi_namespace_node *node = NULL;
 
     node = (struct acpi_namespace_node *)handle;
@@ -570,7 +538,9 @@ KCL_ACPI_ContextHandle ATI_API_CALL KCL_ACPI_GetNotifyContext(KCL_ACPI_DevHandle
 #endif
     /* No alternative on 2.6.29 and later for now. The internal ACPI data
      * structures are no longer included in the kernel headers. 
-     * For 2.6.29 and later, this is not used. Return null here.   */
+     * For 2.6.29 and later, this is not used. Return null here.
+     * And for 2.6.9, there is no ACPI video module, so also return null
+     */
     {
         return NULL;
     }
@@ -622,20 +592,6 @@ void ATI_API_CALL KCL_ACPI_UpdateNotifyContext(KCL_ACPI_DevHandle handle, KCL_AC
     return;
 }
 
-int ATI_API_CALL KCL_ACPI_GetFgConsole(void)
-{
-    return fg_console;
-}
-
-int ATI_API_CALL KCL_ACPI_GetNewVt(int console)
-{
-#if LINUX_VERSION_CODE > KERNEL_VERSION(2,6,11)
-    return vc_cons[console].d->vt_newvt;
-#else
-    return vt_cons[console]->vt_newvt;
-#endif
-}
-
 /** \brief get child device in ACPI namespace
  * \param handle acpi handle for parent device
  * \return acpi handle for child device
@@ -709,6 +665,160 @@ int ATI_API_CALL KCL_ACPI_Cmp_BusID(KCL_ACPI_DevHandle handle, KCL_ACPI_BusId bu
     return (ret);
 }
 
+static unsigned int KCL_ACPI_videoDevice(KCL_ACPI_DevHandle handle)
+{
+    KCL_ACPI_DevHandle dummy_handle;
+
+    if ((KCL_ACPI_GetDevHandle(handle, "_DOD", &dummy_handle) == KCL_ACPI_OK)
+       || (KCL_ACPI_GetDevHandle(handle, "_DOS", &dummy_handle) == KCL_ACPI_OK))
+    {
+        return KCL_ACPI_OK;
+    }
+    else
+    {
+        return KCL_ACPI_ERROR;
+    }
+}
+
+static unsigned int KCL_ACPI_acDevice(KCL_ACPI_DevHandle handle)
+{
+    KCL_ACPI_DevHandle dummy_handle;
+
+    if (KCL_ACPI_GetDevHandle(handle, "_PSR", &dummy_handle) == KCL_ACPI_OK)
+    {
+        return KCL_ACPI_OK;
+    }
+    else
+    {
+        return KCL_ACPI_ERROR;
+    }
+}
+
+static unsigned int KCL_ACPI_PowerXpressDevice(KCL_ACPI_DevHandle handle)
+{
+    KCL_ACPI_DevHandle dummy_handle;
+
+    if (KCL_ACPI_GetDevHandle(handle, "ATPX", &dummy_handle) == KCL_ACPI_OK)
+    {
+        return KCL_ACPI_OK;
+    }
+    else
+    {
+        return KCL_ACPI_ERROR;
+    }
+}
+
+/* call back function for acpi_get_devices */
+static unsigned int KCL_ACPI_SearchHandles(KCL_ACPI_DevHandle handle, unsigned int level, void* context, void **retval)
+{
+#if LINUX_VERSION_CODE <= KERNEL_VERSION(2,6,12)
+    acpi_status status;
+    struct acpi_device *tdev;
+    struct firegl_acpi_pci_data    *pdata = NULL;
+    struct acpi_namespace_node     *node;
+    union acpi_operand_object      *obj_desc;
+#endif
+    struct KCL_ACPI_MatchInfo *pInfo = (struct KCL_ACPI_MatchInfo *)context;
+
+    if ( pInfo->ac_handle == NULL && KCL_ACPI_acDevice(handle) == KCL_ACPI_OK )
+    {
+        pInfo->ac_handle = handle;
+    }
+
+    //Hack: search video handle for old kernels.  
+#if LINUX_VERSION_CODE <= KERNEL_VERSION(2,6,12)
+    if ( pInfo->video_handle == NULL &&  (KCL_ACPI_videoDevice(handle) == KCL_ACPI_OK) )
+    {
+        status = acpi_bus_get_device(handle, &tdev);
+        if ( ACPI_FAILURE(status) )
+        {
+            KCL_DEBUG_ERROR ("Could not find acpi device for video handle: %p, status: 0x%x.\n", handle, status);
+            return KCL_ACPI_OK;
+        }
+
+        node = (struct acpi_namespace_node *)handle;
+        if ( node != NULL && node->object != NULL )
+        {
+            obj_desc = node->object;
+            while (obj_desc) 
+            {
+                if ( (ACPI_GET_OBJECT_TYPE (obj_desc) == ACPI_TYPE_LOCAL_DATA) &&
+                     obj_desc->data.handler != NULL ) 
+                {
+                    // Only two kinds of data is attached to a video device: "struct acpi_device" and "struct acpi_pci_data" 
+                    if ( obj_desc->data.pointer != NULL && (void *)obj_desc->data.pointer != (void *)tdev )
+                    {
+                        pdata = (struct firegl_acpi_pci_data *)obj_desc->data.pointer;
+                        if ( pdata && pdata->dev == pInfo->pcidev )
+                        {
+                            pInfo->video_handle = handle;
+                        }
+                        break;
+                    }
+                }
+                obj_desc = obj_desc->common.next_object;
+            }
+        }
+    }
+    if (pInfo->video_handle)
+    {
+#endif    
+        if (pInfo->ac_handle)
+        {
+            return AE_CTRL_TERMINATE;
+        }
+#if LINUX_VERSION_CODE <= KERNEL_VERSION(2,6,12)
+    }
+#endif
+    return KCL_ACPI_OK;
+}
+
+unsigned int ATI_API_CALL KCL_ACPI_GetHandles(kcl_match_info_t *pInfo)
+{
+#if LINUX_VERSION_CODE > KERNEL_VERSION(2,6,12)
+    #if LINUX_VERSION_CODE > KERNEL_VERSION(2,6,19)
+        pInfo->video_handle = pInfo->pcidev->dev.archdata.acpi_handle;
+    #else 
+        pInfo->video_handle = pInfo->pcidev->dev.firmware_data;
+    #endif    
+    if ( pInfo->video_handle &&
+        (KCL_ACPI_videoDevice(pInfo->video_handle) != KCL_ACPI_OK) )
+    {
+        KCL_DEBUG1(FN_FIREGL_KCL, "Video handle doesn't have ACPI function: %p for video device: %p\n", pInfo->video_handle, pInfo->pcidev);
+        pInfo->video_handle = NULL;
+    }
+#endif
+    return acpi_get_devices(NULL, (acpi_walk_callback)KCL_ACPI_SearchHandles, pInfo, NULL);
+}
+
+static unsigned int KCL_ACPI_SearchAlternateHandle(KCL_ACPI_DevHandle handle, unsigned int level, void* context, void **retval)
+{
+    /*This function is only used by PowerXpress handle search for now*/
+    KCL_ACPI_DevHandle pHandle = (KCL_ACPI_DevHandle) context;
+    if ((KCL_ACPI_PowerXpressDevice(handle) == KCL_ACPI_OK) && handle != pHandle)
+    {
+        *retval = handle;
+        return AE_CTRL_TERMINATE;
+    }
+    return AE_OK;
+}
+
+/** \brief get a different video handle in ACPI namespace
+ * \param handle acpi handle for current video handle
+ * \return alternate acpi video handle
+ */
+KCL_ACPI_DevHandle ATI_API_CALL KCL_ACPI_GetAlternateHandle(KCL_ACPI_DevHandle pHandle)
+{
+	acpi_status status;
+    KCL_ACPI_DevHandle retHandle = NULL;
+    status = acpi_get_devices(NULL, (acpi_walk_callback)KCL_ACPI_SearchAlternateHandle, pHandle, &retHandle);
+    if ( ACPI_FAILURE(status) )
+    {
+        return NULL;
+    }
+    return retHandle;
+}
+
 #else
 
 /*
@@ -723,12 +833,6 @@ unsigned int ATI_API_CALL KCL_ACPI_GetDevHandle(KCL_ACPI_DevHandle parent,
     return KCL_ACPI_NOT_AVAILABLE;
 }
 
-unsigned int ATI_API_CALL KCL_ACPI_GetDevices(KCL_ACPI_CallbackHandle callback,
-                                              KCL_ACPI_ContextHandle context)
-{
-    return KCL_ACPI_NOT_AVAILABLE;
-}
-
 unsigned int ATI_API_CALL KCL_ACPI_EvalObject(KCL_ACPI_DevHandle handle,
                                               struct KCL_ACPI_MethodInputInfo *info)
 {
@@ -750,16 +854,20 @@ unsigned int ATI_API_CALL KCL_ACPI_RemoveHandler(KCL_ACPI_DevHandle device,
     return KCL_ACPI_NOT_AVAILABLE;
 }
 
+unsigned int ATI_API_CALL KCL_ACPI_InstallLidHandler()
+{
+    return KCL_ACPI_NOT_AVAILABLE;
+}
+unsigned int ATI_API_CALL KCL_ACPI_RemoveLidHandler();
+{
+    return KCL_ACPI_NOT_AVAILABLE;
+}
+
 int ATI_API_CALL KCL_ACPI_Disabled(void)
 {
     return (1); /* Needs to mimic acpi_disabled value when permanently disabled */
 }
 
-unsigned int KCL_ACPI_GetAcpiHandles(KCL_ACPI_DevHandle handle, unsigned int level, KCL_ACPI_ContextHandle context, void **retval)
-{
-    return KCL_ACPI_NOT_AVAILABLE;
-}
-
 void KCL_ACPI_AcNotify(KCL_ACPI_DevHandle handle, unsigned int event, KCL_ACPI_ContextHandle data)
 {
     return;
@@ -805,5 +913,39 @@ void KCL_ACPI_VideoNotify(KCL_ACPI_DevHandle handle, unsigned int event, KCL_ACP
     return NULL;
 }
 
+int ATI_API_CALL KCL_ACPI_Cmp_BusID(KCL_ACPI_DevHandle handle, KCL_ACPI_BusId bus_id)
+{
+    return (1);
+}
+
+unsigned int ATI_API_CALL KCL_ACPI_GetHandles(struct KCL_ACPI_MatchInfo *pInfo)
+{
+    return KCL_ACPI_NOT_AVAILABLE;
+}
+
+KCL_ACPI_DevHandle ATI_API_CALL KCL_ACPI_GetAlternateHandle(KCL_ACPI_DevHandle pHandle)
+{
+    return NULL;
+}
+
 #endif
 
+/*
+ * The following KCL ACPI functions need to always work regardless
+ * of whether or not ACPI support has been built into the kernel.
+ */
+
+int ATI_API_CALL KCL_ACPI_GetFgConsole(void)
+{
+    return fg_console;
+}
+
+int ATI_API_CALL KCL_ACPI_GetNewVt(int console)
+{
+#if LINUX_VERSION_CODE > KERNEL_VERSION(2,6,11)
+    return vc_cons[console].d->vt_newvt;
+#else
+    return vt_cons[console]->vt_newvt;
+#endif
+}
+
