@@ -69,12 +69,19 @@ struct KCL_ACPI_MethodInputInfo
     unsigned int padding[9];
 };
 
+//short struct to ease parameter passing
+typedef struct KCL_ACPI_MatchInfo{
+     KCL_ACPI_DevHandle ac_handle; //acpi handle for ac adapter
+     KCL_ACPI_DevHandle video_handle; //acpi handle for video bus
+     struct pci_dev* pcidev;
+} kcl_match_info_t;
+
 extern unsigned int ATI_API_CALL KCL_ACPI_GetDevHandle(KCL_ACPI_DevHandle parent,
                                                        const char *pathname,
                                                        KCL_ACPI_DevHandle *ret_handle);
 
-extern unsigned int ATI_API_CALL KCL_ACPI_GetDevices(KCL_ACPI_CallbackHandle callback,
-                                                     KCL_ACPI_ContextHandle context);
+unsigned int ATI_API_CALL KCL_ACPI_GetHandles(struct KCL_ACPI_MatchInfo *pInfo);
+KCL_ACPI_DevHandle ATI_API_CALL KCL_ACPI_GetAlternateHandle(KCL_ACPI_DevHandle pHandle);
 
 extern unsigned int ATI_API_CALL KCL_ACPI_EvalObject(KCL_ACPI_DevHandle handle,
                                                      struct KCL_ACPI_MethodInputInfo *info);
@@ -87,7 +94,8 @@ extern unsigned int ATI_API_CALL KCL_ACPI_InstallHandler(KCL_ACPI_DevHandle devi
 extern unsigned int ATI_API_CALL KCL_ACPI_RemoveHandler(KCL_ACPI_DevHandle device,
                                                         unsigned int handler_type,
                                                         KCL_ACPI_CallbackHandle handler);
-
+extern unsigned int ATI_API_CALL KCL_ACPI_InstallLidHandler(void);
+extern unsigned int ATI_API_CALL KCL_ACPI_RemoveLidHandler(void);
 extern int ATI_API_CALL KCL_ACPI_Disabled(void);
 extern void ATI_API_CALL KCL_ACPI_ExecHandler(KCL_ACPI_CallbackHandle handler, KCL_ACPI_DevHandle handle, unsigned int event, KCL_ACPI_ContextHandle data);
 extern KCL_ACPI_CallbackHandle ATI_API_CALL KCL_ACPI_GetNotifyHandler(KCL_ACPI_DevHandle handle);
@@ -99,15 +107,15 @@ extern KCL_ACPI_DevHandle ATI_API_CALL KCL_ACPI_GetPeerDevice(KCL_ACPI_DevHandle
 extern int ATI_API_CALL KCL_ACPI_Cmp_BusID(KCL_ACPI_DevHandle handle, KCL_ACPI_BusId bus_id);
 
 // Callback functions called by kernel
-extern unsigned int KCL_ACPI_GetAcpiHandles(KCL_ACPI_DevHandle handle, unsigned int level, KCL_ACPI_ContextHandle context, void **retval);
 extern void KCL_ACPI_AcNotify(KCL_ACPI_DevHandle handle, unsigned int event, KCL_ACPI_ContextHandle data);
 extern void KCL_ACPI_VideoNotify(KCL_ACPI_DevHandle handle, unsigned int event, KCL_ACPI_ContextHandle data);
 
 // libip functions
-extern unsigned int ATI_API_CALL libip_get_acpi_handles(KCL_ACPI_DevHandle handle, unsigned int level, KCL_ACPI_ContextHandle context);
 extern void ATI_API_CALL libip_ac_notify(KCL_ACPI_DevHandle handle, unsigned int event, KCL_ACPI_ContextHandle data, unsigned int *newstate);
 extern void ATI_API_CALL libip_video_notify(KCL_ACPI_DevHandle handle, unsigned int event, KCL_ACPI_ContextHandle data);
 
+extern void ATI_API_CALL libip_lid_open_notify(void);
+
 extern int ATI_API_CALL KCL_ACPI_GetFgConsole(void);
 extern int ATI_API_CALL KCL_ACPI_GetNewVt(int console);
 
